#!/usr/bin/php
<?php
define('PLUGIN_NAME', 'check_api_extreme_networks_ap.php');

// Require auth file DEFAULT_AUTH_FILE or param -A
// Format:
// username="<user>"
// password="<pwd>"

define('SHORT_OPTS', 'hDH:A:T:w:c:J');
define('LONG_OPTS', array('help', 'debug', 'host:', 'authfile:', 'type:', 'warning:', 'critical:', 'json'));
define('STR_STATUS', array('OK', 'WARNING', 'CRITICAL', 'UNKNOWN'));
define('PRIORITIES_STATES', array(2, 1, 3, 0));

define('CLASS_HTTP_CLIENT', '/usr/local/esia/plugins/lib/HttpClient.class.php');
define('CLASS_CACHE_FILE', '/usr/local/esia/plugins/lib/CacheFile.class.php');

define('API_URL', 'https://api.extremecloudiq.com/');
define('DEFAULT_AUTH_FILE', '/etc/esia/extreme_networks.conf');
define('DEFAULT_CACHE_DIR_PATH', '/tmp/esia/extreme_networks/');
define('DEFAULT_CURLOPT_HTTPHEADER', array(
	'content-type: application/json; charset=utf-8',
));

define('AUTH_TOKEN_VALIDITY_GAP', 600); // Seconds (1 min). Gap before token expiration (The token expiration value is given by the api. This gap is subtracted from the value)
define('HTTP_CONNECTION_TIMEOUT', 30); // The number of seconds to wait while trying to connect to the API
define('LOCK_FILE_DURATION', 5); // Minutes to let the lock cache file valid (must be lower than CACHE_DEVICES_VALIDITY). If the lock cache file is valid, another process cannot regenerate the devices data. When a process is regeneration the devices data it regenerate first a lock cache file with this duration.
define('CACHE_DEVICES_VALIDITY', 6); // Cache during in minutes for devices list
define('CACHE_DEVICE_PERF_VALIDITY', 5); // Cache during in minutes for device perfs (cpu/mem)
define('DEFAULT_TEST_INTERVAL', 300); // Seconds (5min)

include_once CLASS_HTTP_CLIENT;
include_once CLASS_CACHE_FILE;

// Print help
function printUsage()
{
    echo 'USAGE : ' . PLUGIN_NAME . " [OPTIONS] [-h to print help]\n\n";
}

function printHelp()
{
    $gap = 20;
    printUsage();
    echo "Check parameters of an Extrem Networks access point connected to the API (ExtremeCloud IQ)\n\n";
    echo "\t-H, " . str_pad('--host=STRING', $gap) . "Device host name\n";
	echo "\t-A, " . str_pad('--authfile=PATH', $gap) . "Auth file path for API login credentials (Format: username=\"<user>\" password=\"<pwd>\")\n";
	echo "\t-T, " . str_pad('--type=STRING', $gap) . "Type of test [cpu, mem, status, clients, inventory] (default: status)\n";
	echo "\t-w, " . str_pad('--warning=INTEGER', $gap) . "Warning threshold in percent\n";
	echo "\t-c, " . str_pad('--critical=INTEGER', $gap) . "Critical threshold in percent\n";
	echo "\t-J, " . str_pad('--json', $gap) . "JSON output for inventory -T inventory required\n";
	echo "\t-D, " . str_pad('--debug', $gap) . "Debug mode\n";
    echo "\n";
    echo "\t-h, " . str_pad('--help', $gap) . "Print this help\n";
    echo "\n";
}

function getParam($options, $shortName, $longName, $default=NULL)
{
    if(isset($options[$shortName]))
    {
        return trim($options[$shortName]);
    }
    elseif(isset($options[$longName]))
    {
        return trim($options[$longName]);
    }

    return $default;
}

// Check if the lock file is valid (another script is refreshing the data). If not, re-create the lock file with new time stamp & duration.
// TRUE => lock file is regenerated and the data can be refreshed.
// FALSE => lock file has been created by another process (will refresh data) and data must not be refreshed by the current process.
function isLockFileReset($file)
{
	$lockCache = new CacheFile($file);
	
	if($lockCache->isValid())
	{
		// Duration time not elapsed. Another process is refreshing data
		return FALSE;
	}

	// Set during in minutes
	$lockCache->setDuring(LOCK_FILE_DURATION);
	// Write data or the file will not be created
	$lockCache->writeData(json_encode('locked'));

	// Lock file time reset. Data can be refreshed
	return TRUE;
}

// Create the http client with connection initialized by token retrieved in cache file or new token request
function initHttpClient($authFile, $cacheFileToken, $isDebug)
{
	try
	{
		$httpClient = NULL;
		$token = '';
		// Get API token from cache or request new token if expired
		$cacheFile = new CacheFile($cacheFileToken);
		if($cacheFile->isValid())
		{
			$cacheData = json_decode($cacheFile->getData(), TRUE);
			if(!empty($cacheData['access_token']))
			{
				$token = $cacheData['access_token'];
			}
		}

		// Request api new auth token if not found in cache or expired
		if(empty($token))
		{
			// Parse config file
			$authHead = parse_ini_file($authFile);
			if(!empty($authHead['username']) && !empty($authHead['password']))
			{
				// Authentication to get access token
				// Instance httpClient for auth
				$httpClientAuth = new HttpClient(API_URL);
				$httpClientAuth->open();
				$httpClientAuth->setoptions(array(
					CURLOPT_CONNECTTIMEOUT => HTTP_CONNECTION_TIMEOUT,
					CURLOPT_HTTPHEADER => DEFAULT_CURLOPT_HTTPHEADER,
					CURLOPT_POSTFIELDS => json_encode($authHead),
				));

				// Send login request
				$authData = $httpClientAuth->post('login');
				$authData = json_decode($authData, TRUE);
				// Auth client is no longer required because the api need to reinstance the client after authentication
				$httpClientAuth->close();
				
				if(!empty($authData['access_token']))
				{
					$token = trim($authData['access_token']);
					// Get token duration validity
					if(!empty($authData['expires_in']))
					{
						// Convert in minutes & keep some gap
						$cacheFile->setDuring(intval((intval($authData['expires_in']) - AUTH_TOKEN_VALIDITY_GAP) / 60));
					}
					// Set the cache file
					$cacheFile->writeData(json_encode(array('access_token' => $token)));
				}
				else
				{
					$status = 3;
					echo STR_STATUS[$status] . " => Failed to authenticate\n";
					exit($status);
				}
			}
			else
			{
				$status = 3;
				echo STR_STATUS[$status] . " => Failed to open auth file or missing parameters (" . $authFile  . ")\n";
				exit($status);
			}
		}

		// Instance httpClient for data fetch. Must be reinstanciated after authentication to avoid extremecloudiq API issue
		$httpClient = new HttpClient(API_URL);
		$httpClient->open();
		// Set header options for api get data
		$curlOptHttpHeader = DEFAULT_CURLOPT_HTTPHEADER;
		$curlOptHttpHeader[] = 'Authorization: Bearer ' . $token;
		$httpClient->setoptions(array(
			CURLOPT_CONNECTTIMEOUT => HTTP_CONNECTION_TIMEOUT,
			CURLOPT_HTTPHEADER => $curlOptHttpHeader,
		));

		return $httpClient;
	}
	catch(CurlLibException | CurlException | Exception $e)
	{
		$status = 3;
		echo STR_STATUS[$status] . " => Cannot initialize API connection\n";
		if($isDebug)
		{
			echo $e->getMessage() . "\n";
		}

		// Don't forget to close curl http clients if failed but client is initialized
		if(isset($httpClientAuth) && is_a($httpClientAuth, 'HttpClient'))
		{
			$httpClientAuth->close();
		}
		if(isset($httpClient) && is_a($httpClient, 'HttpClient'))
		{
			$httpClient->close();
		}

		exit($status);
	}
}

// Get perf data (cpu/mem) from cache file or API
function getPerfData($httpClient, $authFile, $cacheFileToken, $confName, $deviceId, $dataKey, $isDebug)
{
	try
	{
		$cacheFilePerf = DEFAULT_CACHE_DIR_PATH . $confName . '_perf_' . $deviceId . '.json';
		$perfData = array();
		// Check for device perf (cpu/mem) cache
		$perfCache = new CacheFile($cacheFilePerf);
		if($perfCache->isValid())
		{
			$perfData = json_decode($perfCache->getData(), TRUE);
		}

		if(!isset($perfData[$dataKey]))
		{
			// Get data from API
			if(!isset($httpClient))
			{
				// Create http client
				$httpClient = initHttpClient($authFile, $cacheFileToken, $isDebug);
			}

			// Get timestamp & interval in milliseconds
			$now = new DateTime();
			$now = $now->getTimestamp() * 1000;
			$interval = DEFAULT_TEST_INTERVAL * 1000;

			$params = array(
				'startTime' => $now - ($interval * 10), // $interval * 10 or sometimes, the API will return no data
				'endTime' => $now,
				'interval' => $interval,
			);
			$avgList = $httpClient->get('devices/' . $deviceId . '/history/cpu-mem', $params);
			$avgList = json_decode($avgList, TRUE);
			$perfData = array_pop($avgList); // Be sure to have the latest values

			// Set during in minutes
			$perfCache->setDuring(CACHE_DEVICE_PERF_VALIDITY);
			// Set the cache file data
			$perfCache->writeData(json_encode($perfData));
		}

		if(isset($perfData[$dataKey]))
		{
			return $perfData[$dataKey];
		}
		else
		{
			return NULL;
		}
	}
	catch(CurlLibException | CurlException | Exception $e)
	{
		$status = 3;
		echo STR_STATUS[$status] . " => Error fetching cpu-mem data\n";
		if($isDebug)
		{
			echo $e->getMessage() . "\n";
		}

		exit($status);
	}
}

$status = 3;
$paramErrors = array();
$options = getopt(SHORT_OPTS, LONG_OPTS);
$httpClient = NULL;

// If help parameter asked, give it and then kill script
if(isset($options['h']) || isset($options['help']))
{
    printHelp();

    exit($status);
}

// Get parameters
$hostName = getParam($options, 'H', 'host');
$authFile = getParam($options, 'A', 'authfile', DEFAULT_AUTH_FILE);
$type = getParam($options, 'T', 'type', 'status');
$warning = getParam($options, 'w', 'warning', NULL);
$critical = getParam($options, 'c', 'critical', NULL);

// Json for inventory type
if(isset($options['J']) || isset($options['json']))
{
    $isJson = TRUE;
}
else
{
	$isJson = FALSE;
}

// Debug mode or not
if(isset($options['D']) || isset($options['debug']))
{
    $isDebug = TRUE;
}
else
{
	$isDebug = FALSE;
}

// Check if parameters are valid
if(empty($hostName))
{
    $paramErrors[] = "ERROR_PARAM[-H]: Missing device host name";
}
if(empty($type))
{
    $paramErrors[] = "ERROR_PARAM[-T]: Missing test type [cpu, mem, status, clients, inventory]";
}
if(empty($authFile) || !is_string($authFile))
{
    $paramErrors[] = "ERROR_PARAM[-A]: Bad configuration auth file path parameter => " . $authFile;
}
elseif(!file_exists($authFile))
{
	$paramErrors[] = "ERROR_PARAM[-A]: Configuration auth file does not exist => " . $authFile;
}
// Warning & critical params
if(isset($warning))
{
	if(is_numeric($warning))
	{
		$warning = intval($warning);
	}
	else
	{
		$paramErrors[] = "ERROR_PARAM[-w]: Warning threshold must be a number";
	}
}
if(isset($critical))
{
	if(is_numeric($critical))
	{
		$critical = intval($critical);
	}
	else
	{
		$paramErrors[] = "ERROR_PARAM[-c]: Critical threshold must be a number";
	}
}

if(!empty($paramErrors))
{
    foreach($paramErrors as $error)
	{
        echo "$error\n";
    }

    exit($status);
}

// Declare files used for this config
// Get config name from its file name to save different cache files matching the auth file name
// Get only file name from path
$confName = substr($authFile, (strrpos($authFile, '/') + 1));
// Remove file extension
$confName = substr($confName, 0, strrpos($confName, '.'));

// Lock file used to check if another process is refreshing the cache
$lockFile = DEFAULT_CACHE_DIR_PATH . $confName . '.lock';
// File used to save the auth token API
$cacheFileToken = DEFAULT_CACHE_DIR_PATH . $confName . '_token.json';
// File used to save devices data
$cacheFileDevices = DEFAULT_CACHE_DIR_PATH . $confName . '_devices.json';

// Check cache files dir or create it
if(!is_dir(DEFAULT_CACHE_DIR_PATH))
{
	// Create cache file dir
	if(!mkdir(DEFAULT_CACHE_DIR_PATH, 0755, TRUE)) // Create folder. TRUE => recursive
	{
		$status = 3;
		echo STR_STATUS[$status] . " => Failed to create cache file directory (" . DEFAULT_CACHE_DIR_PATH  . ")\n";
		exit($status);
	}
}

// Get devices data cache or refresh it from API
$devicesData = array(
	'total_count' => 0,
	'devices' => array(),
);
try
{
	// Init cache file data
	$devicesCache = new CacheFile($cacheFileDevices);
	
	if($devicesCache->isValid())
	{
		$devicesData = json_decode($devicesCache->getData(), TRUE);
	}
	else
	{
		// Refresh devices data if not found in cache or expired & lock file is not used by another process
		// Check and re-create lock file
		$isCanRefresh = isLockFileReset($lockFile);

		if(!$isCanRefresh)
		{
			// Data cannot be refreshed (another process is using it). Use expired devices data to not block the script
			$devicesData = json_decode($devicesCache->getData(), TRUE);
		}
		else
		{
			// Refresh data
			// Create http client
			$httpClient = initHttpClient($authFile, $cacheFileToken, $isDebug);

			// Get all devices from api recursively because the api has a pagination where the limit is 100 devices max
			// Available views : BASIC, FULL, STATUS, LOCATION, CLIENT, DETAIL
			$params = array(
				'page' => 1,
				'limit' => 100,
				'views' => 'BASIC,CLIENT,LOCATION',
			);
			
			$hasMoreData = TRUE;
			while($hasMoreData)
			{
				$resp = $httpClient->get('devices', $params);
				$resp = @json_decode($resp, TRUE);

				if(!empty($resp['data']))
				{
					// Create an array where the key is the hostname so the next scripts using the cache file will find it faster in the list
					foreach($resp['data'] as $dev)
					{
						$devicesData['devices'][$dev['hostname']] = $dev;
					}

					// Get the total devices count
					if(isset($resp['total_count']))
					{
						$devicesData['total_count'] = $resp['total_count'];
					}

					if(isset($resp['count']) && $resp['count'] < $params['limit'])
					{
						// It was the last page
						$hasMoreData = FALSE;
					}
				}
				else
				{
					$hasMoreData = FALSE;
				}

				$params['page']++;
			}

			// Set during in minutes
			$devicesCache->setDuring(CACHE_DEVICES_VALIDITY);
			// Set the cache file data
			$devicesCache->writeData(json_encode($devicesData));

			// Delete lock file for safety if the lock file duration end after the device cache duration
			@unlink($lockFile);
		}
	}
}
catch(CurlLibException | CurlException | Exception $e)
{
	$status = 3;
	echo STR_STATUS[$status] . " => Error fetching devices list\n";
	if($isDebug)
	{
		echo $e->getMessage() . "\n";
	}

	if(isset($httpClient) && is_a($httpClient, 'HttpClient'))
	{
		$httpClient->close();
	}

	exit($status);
}

// Get device to test
if(isset($devicesData['devices'][$hostName]))
{
	$device = $devicesData['devices'][$hostName];
	$message = '';
	$rrd = '';

	// Test types
	if($type == 'cpu')
	{
		// CPU
		$cpu = getPerfData($httpClient, $authFile, $cacheFileToken, $confName, $device['id'], 'average_cpu', $isDebug);

		if(!is_null($cpu))
		{
			$cpu = intval($cpu);
			$rrd = '|cpu=' . $cpu;
			if(isset($critical) && $cpu >= $critical)
			{
				$status = 2;
				$message = "CPU usage: $cpu% (>= $critical)";
			}
			elseif(isset($warning) && $cpu >= $warning)
			{
				$status = 1;
				$message = "CPU usage: $cpu% (>= $warning)";
			}
			else
			{
				$status = 0;
				$message = "CPU usage: $cpu%";
			}

			// Write output
			echo STR_STATUS[$status] . " => " . $message . $rrd . "\n";
		}
		else
		{
			$status = 3;
			echo STR_STATUS[$status] . " => Cannot get cpu data\n";
		}
	}
	elseif($type == 'mem')
	{
		// Memory
		$mem = getPerfData($httpClient, $authFile, $cacheFileToken, $confName, $device['id'], 'average_memory', $isDebug);

		if(!is_null($mem))
		{
			$mem = intval($mem);
			$rrd = '|mem=' . $mem;
			if(isset($critical) && $mem >= $critical)
			{
				$status = 2;
				$message = "Memory usage: $mem% (>= $critical)";
			}
			elseif(isset($warning) && $mem >= $warning)
			{
				$status = 1;
				$message = "Memory usage: $mem% (>= $warning)";
			}
			else
			{
				$status = 0;
				$message = "Memory usage: $mem%";
			}

			// Write output
			echo STR_STATUS[$status] . " => " . $message . $rrd . "\n";
		}
		else
		{
			$status = 3;
			echo STR_STATUS[$status] . " => Cannot get memory data\n";
		}
	}
	elseif($type == 'status')
	{
		// AP connected state
		$allStates = array();
		// Connected
		if(empty($device['connected']))
		{
			$allStates[] = 2;
			echo STR_STATUS[2] . " => $hostName is disconnected\n";
		}
		else
		{
			$allStates[] = 0;
			echo STR_STATUS[0] . " => $hostName is connected\n";
		}
		// Config
		if(empty($device['config_mismatch']))
		{
			$allStates[] = 0;
			echo STR_STATUS[0] . " => Configuration audit match\n";
		}
		else
		{
			$allStates[] = 1;
			echo STR_STATUS[1] . " => Configuration audit mismatch\n";
		}
		// Location info
		$locMessage = 'Not found';
		if(isset($device['locations']))
		{
			$locations = array();
			foreach($device['locations'] as $loc)
			{
				$locations[] = trim($loc['name']);
			}
			if(!empty($locations))
			{
				$locMessage = implode(' > ', $locations);
			}
		}
		echo "Device location: $locMessage\n";

		// Get the most priority state
		if(!empty($allStates))
		{
			foreach(PRIORITIES_STATES as $stateCode)
			{
				if(in_array($stateCode, $allStates))
				{
					$status = $stateCode;
					break;
				}
			}
		}
		else
		{
			$status = 3;
		}
	}
	elseif($type == 'clients')
	{
		// Clients connected count
		$clients = 0;
		if(isset($device['active_clients']) && is_numeric($device['active_clients']))
		{
			$clients = intval($device['active_clients']);
		}

		$rrd = '|clients=' . $clients;
		if(isset($critical) && $clients >= $critical)
		{
			$status = 2;
			$message = "Clients connected: $clients (>= $critical)";
		}
		elseif(isset($warning) && $clients >= $warning)
		{
			$status = 1;
			$message = "Clients connected: $clients (>= $warning)";
		}
		else
		{
			$status = 0;
			$message = "Clients connected: $clients";
		}

		// Write output
		echo STR_STATUS[$status] . " => " . $message . $rrd . "\n";
	}
	elseif($type == 'inventory')
	{
		// Data for esia inventory webp
		$inv = array();
		// Device
		$inv[] = array(
			'name' => 'Extreme Networks - ' . $device['product_type'],
			'manufacturer' => 'Extreme Networks',
			'model' => $device['product_type'],
			'type' => 'CHASSIS',
			'serial' => $device['serial_number'],
			'hardwareVersion' => '',
			'firmwareVersion' => $device['software_version'],
		);
		// Interface
		$inv[] = array(
			'name' => 'ETH',
			'fullname' => '',
			'manufacturer' => 'Extreme Networks',
			'model' => '',
			'type' => 'NETINT',
			'hardwareVersion' => '',
			'firmwareVersion' => '',
			'option' => array(
				'macAdress' => $device['mac_address'],
				'IP' => $device['ip_address'],
			),
		);

		if($isJson)
		{
			echo json_encode($inv);
		}
		else
		{
			// Device
			echo 'name: ' . $inv[0]['name'] . ', manufacturer: ' . $inv[0]['manufacturer'] . ', model: ' . $inv[0]['model'] . ', type: ' . $inv[0]['type'] . ', serial: ' . $inv[0]['serial'] . ', hardwareVersion: ' . $inv[0]['hardwareVersion'] . ', firmwareVersion: ' . $inv[0]['firmwareVersion'] . "\n";
			// Interface
			echo 'name: ' . $inv[1]['name'] . ', fullname: ' . $inv[1]['fullname'] . ', manufacturer: ' . $inv[1]['manufacturer'] . ', model: ' . $inv[1]['model'] . ', type: ' . $inv[1]['type'] . ', hardwareVersion: ' . $inv[1]['hardwareVersion'] . ', firmwareVersion: ' . $inv[1]['firmwareVersion'] . ', macAdress: ' . $inv[1]['option']['macAdress'] . ', IP: ' . $inv[1]['option']['IP'] . "\n";
		}
		exit(0);
	}
	else
	{
		$status = 3;
		echo STR_STATUS[$status] . " => Unknown test type -T [cpu, mem, status, clients, inventory]\n";
	}
}
else
{
	$status = 3;
	echo STR_STATUS[$status] . " => Device not found (" . $hostName . ")\n";
}

// Don't forget to close curl http client
if(isset($httpClient) && is_a($httpClient, 'HttpClient'))
{
	$httpClient->close();
}

exit($status);
?>
