#!/usr/bin/perl
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
#use lib "/root/libperl/"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Math::Round;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::CacheFile;
use ESIA::INVENTORY;
use JSON;


my $PROGNAME='check_snmp_stormshield_version.pl';
my $PROGDESC = "Check Stormshield by SNMP";

my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

ESIA::PLUGIN::arm_end_signal($TIMEOUT);
my $return=0;
my $type='cpu';
my ($opt_h,$opt_V,$json);
my $warning=80;
my $critical=90;
my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";
my $oid_model 	= ".1.3.6.1.4.1.11256.1.0.1.0";
my $oid_version = ".1.3.6.1.4.1.11256.1.0.2.0";
my $oid_SN 		= ".1.3.6.1.4.1.11256.1.0.3.0";
my $oid_name 	= ".1.3.6.1.4.1.11256.1.0.4.0";
my $oid_proc_load = "1.3.6.1.2.1.25.3.3.1.2"; # %time the proc was not idle over last minute

my $oid_ram_free	= "1.3.6.1.4.1.2021.4.6.0";  # Real memory free
my $oid_ram_total	= "1.3.6.1.4.1.2021.4.5.0";  # Real memory total
my $oid_ram_cache   = "1.3.6.1.4.1.2021.4.15.0"; # Real memory cached
my $oid_swap_free	= "1.3.6.1.4.1.2021.4.4.0";  # swap memory free
my $oid_swap_total	= "1.3.6.1.4.1.2021.4.3.0";  # Swap memory total

my $oid_IPSECStatsSPDIn='.1.3.6.1.4.1.11256.1.13.1.1.0';
my $oid_IPSECStatsSPDOut='.1.3.6.1.4.1.11256.1.13.1.2.0';
my $oid_IPSECStatsSADLarval='.1.3.6.1.4.1.11256.1.13.2.1.0';
my $oid_IPSECStatsSADMature='.1.3.6.1.4.1.11256.1.13.2.2.0';
my $oid_IPSECStatsSADDying='.1.3.6.1.4.1.11256.1.13.2.3.0';
my $oid_IPSECStatsSADDead='.1.3.6.1.4.1.11256.1.13.2.4.0';



sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
#
#MAIN
#
&Getopt::Long::config('bundling');
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"T=s" => \$type,"type=s" => \$type,
	"w=s" => \$warning,"warning=s" => \$warning,
	"c=s" => \$critical,"critical=s" => \$critical,
	"J" => \$json,	 "json" => \$json
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-T","--type","type of check [cpu|mem|version|module|vpn] (default: cpu)" ]);
	push(@arr,["-w","--warning","warning threshold in percent (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent(default: 90) "]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}

if($type eq 'cpu')
{
	my $result = $snmp->get_table( $oid_proc_load);
	my $load=0;
	my $nbrcpu = 0;
    foreach my $key ( keys %$result)
    {
    	$load=$load + $$result{$key};
    	$nbrcpu++;
	}
	if($nbrcpu == 0)
	{
		print "UNKNOWN: No CPU information found\n";
		exit $ERRORS{'UNKNOWN'};
	}
	else
	{
		$load = $load / $nbrcpu;
		$load = sprintf("%.1f",$load);
		my $state = "OK";
		my $message = "< $warning%";
		if($load < $warning)
		{
			$state = "OK";
		}
		elsif($load < $critical)
		{
			$state = "WARNING";
			$message = "> $warning%";
		}
		else
		{
			$state = "CRITICAL";
			$message = "> $critical%";
		}

		print "$state: $nbrcpu CPU, average load $load% $message\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	}
}
elsif($type eq 'mem')
{
	my $ram_free = ESIA::SNMP::get($snmp,"$oid_ram_free");
	if(!defined($ram_free))
	{
		print "UNKNOWN: Host not SNMP response $snmperror\n";
		exit $ERRORS{'UNKNOWN'};
	}
	my $ram_cache = ESIA::SNMP::get($snmp,"$oid_ram_cache");
	my $ram_total = ESIA::SNMP::get($snmp,"$oid_ram_total");
	my $ram_used = undef;
	if(defined($ram_cache))
	{
		$ram_used = $ram_total-$ram_free - $ram_cache;
	}
	else
	{
		$ram_used = $ram_total-$ram_free;
	}
	my $percent_used = ($ram_used/$ram_total)*100;
	$percent_used = sprintf("%.0f",$percent_used);
	my $ram_used_mb = sprintf("%.0f",$ram_used/1024);
	my $ram_total_mb = sprintf("%.0f",$ram_total/1024);
	my $state = "OK";
	my $message = "< ".$warning;
	if($percent_used < $warning)
	{
		$state = "OK";
	}
	elsif($percent_used < $critical)
	{
		$state = "WARNING";
		$message = ">".$warning;
	}
	else
	{
		$state = "CRITICAL";
		$message = "> ".$critical;
	}
	print "$state: Ram : $percent_used% (".$ram_used_mb."MB/".$ram_total_mb."MB) $message%\n";
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "|ram=$percent_used;ramByte=$ram_used;ramTotal=$ram_total;";
}
elsif($type eq 'vpn')
{
	my ($min_warn, $max_warn) = split(/,/,$warning);
	my ($min_crit, $max_crit) = split(/,/,$critical);
	my $larval=ESIA::SNMP::get($snmp,$oid_IPSECStatsSADLarval);
	my $mature=ESIA::SNMP::get($snmp,$oid_IPSECStatsSADMature);
	my $dying=ESIA::SNMP::get($snmp,$oid_IPSECStatsSADDying);
	my $dead=ESIA::SNMP::get($snmp,$oid_IPSECStatsSADDead);

	my $message = "$mature matures/running VPNs + ($larval larval, $dying dying, $dead dead)|$min_crit,$min_warn,$max_warn,$max_crit";

	my $status="OK";
	if( defined($max_crit) && $mature >= $max_crit )
	{
		$status = "CRITICAL";
		print "$status - $message\n";
	}
	elsif( $mature <= $min_crit )
	{
		$status = "CRITICAL";
		print "$status - $message\n";
	}
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$status});
	if( defined($max_warn) && $mature >= $max_warn )
	{
		$status = "WARNING";
		print "$status - $message\n";
	}
	elsif( $mature <= $min_warn )
	{
		$status = "WARNING";
		print "$status - $message\n";
	}
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$status});
}
elsif($type eq 'version' || $type eq 'inventory')
{
	my $model = ESIA::SNMP::get($snmp,"$oid_model");
	my $version = ESIA::SNMP::get($snmp,"$oid_version");
	my $serial = ESIA::SNMP::get($snmp,"$oid_SN");
	my $name = ESIA::SNMP::get($snmp,"$oid_name");
	my @output;
	my %hash = ('name' => $name,'manufacturer'=>'Stormshield', 'model' => $model, 'type'=>'chassis','serial'=>$serial,'hardwareVersion' => '', 'firmwareVersion' => $version);
	push(@output, \%hash);
	if($json)
	{
		my $out = encode_json(\@output);
		print $out;
	}
	else
	{
	    my $size = @output;
	    if($size != 0)
	    {
	        foreach my $line ( @output)
	        {
	            my $hash = \%$line;
	            print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
	            print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
	        }
	    }
	}
}

exit($return);
